@@ -1,20 +1,20 @@
 '''
 Implements the RTS generic Target fabric classes.
 
-This file is part of RTSLib Community Edition.
-Copyright (c) 2011 by RisingTide Systems LLC
+This file is part of RTSLib.
+Copyright (c) 2011-2013 by Datera, Inc
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as
-published by the Free Software Foundation, version 3 (AGPLv3).
+Licensed under the Apache License, Version 2.0 (the "License"); you may
+not use this file except in compliance with the License. You may obtain
+a copy of the License at
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+    http://www.apache.org/licenses/LICENSE-2.0
 
-You should have received a copy of the GNU Affero General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+License for the specific language governing permissions and limitations
+under the License.
 '''
 
 import re
@@ -24,6 +24,7 @@ import uuid
 import shutil
 
 from node import CFSNode
+from os.path import isdir
 from doctest import testmod
 from configobj import ConfigObj
 from utils import RTSLibError, RTSLibBrokenLink, modprobe
@@ -35,7 +36,7 @@ class FabricModule(CFSNode):
     This is an interface to RTS Target Fabric Modules.
     It can load/unload modules, provide information about them and
     handle the configfs housekeeping. It uses module configuration
-    files in /var/target/fabric/*.spec. After instanciation, whether or
+    files in /var/target/fabric/*.spec. After instantiation, whether or
     not the fabric module is loaded and
     '''
 
@@ -46,7 +47,7 @@ class FabricModule(CFSNode):
     # FabricModule private stuff
     def __init__(self, name):
         '''
-        Instanciate a FabricModule object, according to the provided name.
+        Instantiate a FabricModule object, according to the provided name.
         @param name: the name of the FabricModule object. It must match an
         existing target fabric module specfile (name.spec).
         @type name: str
@@ -150,7 +151,9 @@ class FabricModule(CFSNode):
         if spec['wwn_from_files']:
             for wwn_pattern in spec['wwn_from_files']:
                 for wwn_file in glob.iglob(wwn_pattern):
-                    wwns_in_file = re.split('\t|\0|\n| ', fread(wwn_file))
+                    wwns_in_file = [wwn for wwn in
+                                    re.split('\t|\0|\n| ', fread(wwn_file))
+                                    if wwn.strip()]
                     if spec['wwn_from_files_filter']:
                         wwns_filtered = []
                         for wwn in wwns_in_file:
@@ -169,7 +172,9 @@ class FabricModule(CFSNode):
         if spec['wwn_from_cmds']:
             for wwn_cmd in spec['wwn_from_cmds']:
                 cmd_result = exec_argv(wwn_cmd, shell=True)
-                wwns_from_cmd = re.split('\t|\0|\n| ', cmd_result)
+                wwns_from_cmd = [wwn for wwn in
+                                 re.split('\t|\0|\n| ', cmd_result)
+                                 if wwn.strip()]
                 if spec['wwn_from_cmds_filter']:
                     wwns_filtered = []
                     for wwn in wwns_from_cmd:
@@ -351,11 +356,11 @@ class LUN(CFSNode):
 
     def __init__(self, parent_tpg, lun, storage_object=None, alias=None):
         '''
-        A LUN object can be instanciated in two ways:
+        A LUN object can be instantiated in two ways:
             - B{Creation mode}: If I{storage_object} is specified, the
               underlying configFS object will be created with that parameter.
               No LUN with the same I{lun} index can pre-exist in the parent TPG
-              in that mode, or instanciation will fail.
+              in that mode, or instantiation will fail.
             - B{Lookup mode}: If I{storage_object} is not set, then the LUN
               will be bound to the existing configFS LUN object of the parent
               TPG having the specified I{lun} index. The underlying configFS
@@ -479,11 +484,13 @@ class LUN(CFSNode):
             return []
         else:
             base = "%s/acls/" % tpg.path
-            xmlun = ["param", "info", "cmdsn_depth", "auth", "attrib"]
+            xmlun = ["param", "info", "cmdsn_depth", "auth", "attrib",
+                     "node_name", "port_name"]
             return [MappedLUN(NodeACL(tpg, nodeacl), mapped_lun.split('_')[1])
                     for nodeacl in listdir(base)
                     for mapped_lun in listdir("%s/%s" % (base, nodeacl))
                     if mapped_lun not in xmlun
+                    if isdir("%s/%s/%s" % (base, nodeacl, mapped_lun))
                     for link in listdir("%s/%s/%s" \
                                         % (base, nodeacl, mapped_lun))
                     if realpath("%s/%s/%s/%s" \
@@ -538,11 +545,11 @@ class MappedLUN(CFSNode):
     def __init__(self, parent_nodeacl, mapped_lun,
                  tpg_lun=None, write_protect=None):
         '''
-        A MappedLUN object can be instanciated in two ways:
+        A MappedLUN object can be instantiated in two ways:
             - B{Creation mode}: If I{tpg_lun} is specified, the underlying
               configFS object will be created with that parameter. No MappedLUN
               with the same I{mapped_lun} index can pre-exist in the parent
-              NodeACL in that mode, or instanciation will fail.
+              NodeACL in that mode, or instantiation will fail.
             - B{Lookup mode}: If I{tpg_lun} is not set, then the MappedLUN will
               be bound to the existing configFS MappedLUN object of the parent
               NodeACL having the specified I{mapped_lun} index. The underlying
@@ -611,9 +618,10 @@ class MappedLUN(CFSNode):
                               % str(tpg_lun))
         os.symlink(tpg_lun.path, "%s/%s"
                    % (self.path, str(uuid.uuid4())[-10:]))
-        if write_protect:
-            self.write_protect = True
-        else:
+
+        try:
+            self.write_protect = int(write_protect) > 0
+        except:
             self.write_protect = False
 
     def _get_alias(self):
@@ -733,70 +741,6 @@ class NodeACL(CFSNode):
     def _get_parent_tpg(self):
         return self._parent_tpg
 
-    def _get_chap_mutual_password(self):
-        self._check_self()
-        path = "%s/auth/password_mutual" % self.path
-        value = fread(path).strip()
-        if value == "NULL":
-            return ''
-        else:
-            return value
-
-    def _set_chap_mutual_password(self, password):
-        self._check_self()
-        path = "%s/auth/password_mutual" % self.path
-        if password.strip() == '':
-            password = "NULL"
-        fwrite(path, "%s" % password)
-
-    def _get_chap_mutual_userid(self):
-        self._check_self()
-        path = "%s/auth/userid_mutual" % self.path
-        value = fread(path).strip()
-        if value == "NULL":
-            return ''
-        else:
-            return value
-
-    def _set_chap_mutual_userid(self, userid):
-        self._check_self()
-        path = "%s/auth/userid_mutual" % self.path
-        if userid.strip() == '':
-            userid = "NULL"
-        fwrite(path, "%s" % userid)
-
-    def _get_chap_password(self):
-        self._check_self()
-        path = "%s/auth/password" % self.path
-        value = fread(path).strip()
-        if value == "NULL":
-            return ''
-        else:
-            return value
-
-    def _set_chap_password(self, password):
-        self._check_self()
-        path = "%s/auth/password" % self.path
-        if password.strip() == '':
-            password = "NULL"
-        fwrite(path, "%s" % password)
-
-    def _get_chap_userid(self):
-        self._check_self()
-        path = "%s/auth/userid" % self.path
-        value = fread(path).strip()
-        if value == "NULL":
-            return ''
-        else:
-            return value
-
-    def _set_chap_userid(self, userid):
-        self._check_self()
-        path = "%s/auth/userid" % self.path
-        if userid.strip() == '':
-            userid = "NULL"
-        fwrite(path, "%s" % userid)
-
     def _get_tcq_depth(self):
         self._check_self()
         path = "%s/cmdsn_depth" % self.path
@@ -805,15 +749,11 @@ class NodeACL(CFSNode):
     def _set_tcq_depth(self, depth):
         self._check_self()
         path = "%s/cmdsn_depth" % self.path
-        fwrite(path, "%s" % depth)
-
-    def _get_authenticate_target(self):
-        self._check_self()
-        path = "%s/auth/authenticate_target" % self.path
-        if fread(path).strip() == "1":
-            return True
-        else:
-            return False
+        try:
+            fwrite(path, "%s" % depth)
+        except IOError, msg:
+            msg = msg[1]
+            raise RTSLibError("Cannot set tcq_depth: %s" % str(msg))
 
     def _list_mapped_luns(self):
         self._check_self()
@@ -850,19 +790,6 @@ class NodeACL(CFSNode):
         return MappedLUN(self, mapped_lun=mapped_lun, tpg_lun=tpg_lun,
                          write_protect=write_protect)
 
-    chap_userid = property(_get_chap_userid, _set_chap_userid,
-                           doc="Set or get the initiator CHAP auth userid.")
-    chap_password = property(_get_chap_password, _set_chap_password,
-                             doc=\
-                             "Set or get the initiator CHAP auth password.")
-    chap_mutual_userid = property(_get_chap_mutual_userid,
-                                  _set_chap_mutual_userid,
-                                  doc=\
-                                  "Set or get the mutual CHAP auth userid.")
-    chap_mutual_password = property(_get_chap_mutual_password,
-                                    _set_chap_mutual_password,
-                                    doc=\
-                                    "Set or get the mutual CHAP password.")
     tcq_depth = property(_get_tcq_depth, _set_tcq_depth,
                          doc="Set or get the TCQ depth for the initiator " \
                          + "sessions matching this NodeACL.")
@@ -870,8 +797,6 @@ class NodeACL(CFSNode):
             doc="Get the parent TPG object.")
     node_wwn = property(_get_node_wwn,
             doc="Get the node wwn.")
-    authenticate_target = property(_get_authenticate_target,
-            doc="Get the boolean authenticate target flag.")
     mapped_luns = property(_list_mapped_luns,
             doc="Get the list of all MappedLUN objects in this NodeACL.")
 
@@ -886,15 +811,15 @@ class NetworkPortal(CFSNode):
 
     # NetworkPortal private stuff
 
-    def __init__(self, parent_tpg, ip_address, port, mode='any'):
+    def __init__(self, parent_tpg, ip_address, port=3260, mode='any'):
         '''
         @param parent_tpg: The parent TPG object.
         @type parent_tpg: TPG
         @param ip_address: The ipv4 IP address of the NetworkPortal.
         @type ip_address: string
-        @param port: The NetworkPortal TCP/IP port.
+        @param port: The optional (defaults to 3260) NetworkPortal TCP/IP port.
         @type port: int
-        @param mode:An optionnal string containing the object creation mode:
+        @param mode: An optionnal string containing the object creation mode:
             - I{'any'} means the configFS object will be either looked up or
               created.
             - I{'lookup'} means the object MUST already exist configFS.
@@ -938,8 +863,40 @@ class NetworkPortal(CFSNode):
     def _get_parent_tpg(self):
         return self._parent_tpg
 
+    def _set_iser_attr(self, iser_attr):
+        path = "%s/iser" % self.path
+        if os.path.isfile(path):
+            if iser_attr:
+                fwrite(path, "1")
+            else:
+                fwrite(path, "0")
+        else:
+            raise RTSLibError("iser network portal attribute does not exist.")
+
+    def _get_iser_attr(self):
+        path = "%s/iser" % self.path
+        if os.path.isfile(path):
+            iser_attr = fread(path).strip()
+            if iser_attr == "1":
+                return True
+            else:
+                return False
+        else:
+            return False
+
     # NetworkPortal public stuff
 
+    def delete(self):
+        '''
+        Delete the NetworkPortal.
+        '''
+        path = "%s/iser" % self.path
+        if os.path.isfile(path):
+            iser_attr = fread(path).strip()
+            if iser_attr == "1":
+                fwrite(path, "0")
+        super(NetworkPortal, self).delete()
+
     parent_tpg = property(_get_parent_tpg,
             doc="Get the parent TPG object.")
     port = property(_get_port,
