@@ -1,20 +1,20 @@
 '''
 Provides various utility functions.
 
-This file is part of RTSLib Community Edition.
-Copyright (c) 2011 by RisingTide Systems LLC
+This file is part of RTSLib.
+Copyright (c) 2011-2013 by Datera, Inc
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as
-published by the Free Software Foundation, version 3 (AGPLv3).
+Licensed under the Apache License, Version 2.0 (the "License"); you may
+not use this file except in compliance with the License. You may obtain
+a copy of the License at
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+    http://www.apache.org/licenses/LICENSE-2.0
 
-You should have received a copy of the GNU Affero General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+License for the specific language governing permissions and limitations
+under the License.
 '''
 
 import re
@@ -48,7 +48,7 @@ class RTSLibBrokenLink(RTSLibError):
 class RTSLibNotInCFS(RTSLibError):
     '''
     The underlying configfs object does not exist. Happens when
-    calling methods of an object that is instanciated but have
+    calling methods of an object that is instantiated but have
     been deleted from congifs, or when trying to lookup an
     object that does not exist.
     '''
@@ -352,6 +352,7 @@ def get_block_type(path):
                               161,  # Carmel 8-port SATA Disks
                               199,  # Veritas volume manager (VxVM) volumes
                               201,  # Veritas VxVM dynamic multipathing driver
+                              230,  # ZFS ZVols
                               240,  # LOCAL/EXPERIMENTAL USE
                               241,  # LOCAL/EXPERIMENTAL USE
                               242,  # LOCAL/EXPERIMENTAL USE
@@ -422,15 +423,14 @@ def convert_scsi_path_to_hctl(path):
     values representing the SCSI ID of the device, or None if no
     match is found.
     '''
-    dev = os.path.realpath(path)
-    scsi_devices = [os.path.basename(scsi_dev).split(':')
-            for scsi_dev in glob.glob("/sys/class/scsi_device/*")]
-    for (host, controller, target, lun) in scsi_devices:
-        scsi_dev = convert_scsi_hctl_to_path(host, controller, target, lun)
-        if dev == scsi_dev:
-            return (int(host), int(controller), int(target), int(lun))
-
-    return None
+    devname = os.path.basename(os.path.realpath(path))
+    try:
+        hctl = os.listdir("/sys/block/%s/device/scsi_device"
+                          % devname)[0].split(':')
+    except:
+        return None
+    
+    return [int(data) for data in hctl]
 
 def convert_scsi_hctl_to_path(host, controller, target, lun):
     '''
@@ -464,28 +464,12 @@ def convert_scsi_hctl_to_path(host, controller, target, lun):
         raise RTSLibError(
             "The host, controller, target and lun parameter must be integers.")
 
-    scsi_dev_path = "/sys/class/scsi_device"
-    sysfs_names = [os.path.basename(name) for name
-            in glob.glob("%s/%d:%d:%d:%d/device/block:*"
-            % (scsi_dev_path, host, controller, target, lun))]
-    if len(sysfs_names) == 0:
-        sysfs_names = [os.path.basename(name) for name
-                in glob.glob("%s/%d:%d:%d:%d/device/block/*"
-                % (scsi_dev_path, host, controller, target, lun))]
-    if len(sysfs_names) > 0:
-        for name in sysfs_names:
-            name1 = name.partition(":")[2].strip()
-            if name1:
-                name = name1
-            dev = os.path.realpath("/dev/%s" % name)
-            try:
-                mode = os.stat(dev)[stat.ST_MODE]
-            except OSError:
-                pass
-            if stat.S_ISBLK(mode):
-                return dev
-    else:
-        return ''
+    for devname in os.listdir("/sys/block"):
+        path = "/dev/%s" % devname
+        hctl = [host, controller, target, lun]
+        if convert_scsi_path_to_hctl(path) == hctl:
+            return os.path.realpath(path)
+    return ''
 
 def convert_human_to_bytes(hsize, kilo=1024):
     '''
@@ -604,8 +588,14 @@ def list_available_kernel_modules():
     '''
     kver = os.uname()[2]
     depfile = "/lib/modules/%s/modules.dep" % kver
-    return [module.split(".")[0] for module in
-            re.findall(r"[a-zA-Z0-9_-]+\.ko:", fread(depfile))]
+    handle = open(depfile)
+    try:
+        lines = handle.readlines()
+    finally:
+        handle.close()
+
+    return [os.path.basename(line.partition(":")[0]).partition(".")[0]
+            for line in lines]
 
 def list_loaded_kernel_modules():
     '''
